@@ -1,10 +1,10 @@
 ;;; exwm.el --- Emacs X Window Manager  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2021 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
-;; Maintainer: Chris Feng <chris.w.feng@gmail.com>
-;; Version: 0.23
+;; Maintainer: Adrián Medraño Calvo <adrian@medranocalvo.com>
+;; Version: 0.25
 ;; Package-Requires: ((xelb "0.18"))
 ;; Keywords: unix
 ;; URL: https://github.com/ch11ng/exwm
@@ -46,7 +46,7 @@
 ;;
 ;;    (require 'exwm)
 ;;    (require 'exwm-config)
-;;    (exwm-config-default)
+;;    (exwm-config-example)
 ;;
 ;; 3. Link or copy the file 'xinitrc' to '~/.xinitrc'.
 ;; 4. Launch EXWM in a console (e.g. tty1) with
@@ -446,16 +446,18 @@
         (when (buffer-live-p buffer)
           (with-current-buffer buffer
             (when (eq exwm--frame exwm-workspace--current)
-              (setq iconic (exwm-layout--iconic-state-p))
-              (when iconic
-                ;; State change: iconic => normal.
-                (set-window-buffer (frame-selected-window exwm--frame)
-                                   (current-buffer)))
-              ;; Focus transfer.
-              (setq window (get-buffer-window nil t))
-              (when (or iconic
-                        (not (eq window (selected-window))))
-                (select-window window)))))))
+              (if exwm--floating-frame
+                  (select-frame exwm--floating-frame)
+                (setq iconic (exwm-layout--iconic-state-p))
+                (when iconic
+                  ;; State change: iconic => normal.
+                  (set-window-buffer (frame-selected-window exwm--frame)
+                                     (current-buffer)))
+                ;; Focus transfer.
+                (setq window (get-buffer-window nil t))
+                (when (or iconic
+                          (not (eq window (selected-window))))
+                  (select-window window))))))))
      ;; _NET_CLOSE_WINDOW.
      ((= type xcb:Atom:_NET_CLOSE_WINDOW)
       (let ((buffer (exwm--id->buffer id)))
@@ -477,7 +479,33 @@
                  )
                 ((= direction xcb:ewmh:_NET_WM_MOVERESIZE_CANCEL)
                  (exwm-floating--stop-moveresize))
-                (t (exwm-floating--start-moveresize id direction))))))
+                ;; In case it's a workspace frame.
+                ((and (not buffer)
+                      (catch 'break
+                        (dolist (f exwm-workspace--list)
+                          (when (or (eq id (frame-parameter f 'exwm-outer-id))
+                                    (eq id (frame-parameter f 'exwm-id)))
+                            (throw 'break t)))
+                        nil)))
+                (t
+                 ;; In case it's a floating frame,
+                 ;; move the corresponding X window instead.
+                 (unless buffer
+                   (catch 'break
+                     (dolist (pair exwm--id-buffer-alist)
+                       (with-current-buffer (cdr pair)
+                         (when
+                             (and exwm--floating-frame
+                                  (or (eq id
+                                          (frame-parameter exwm--floating-frame
+                                                           'exwm-outer-id))
+                                      (eq id
+                                          (frame-parameter exwm--floating-frame
+                                                           'exwm-id))))
+                           (setq id exwm--id)
+                           (throw 'break nil))))))
+                 ;; Start to move it.
+                 (exwm-floating--start-moveresize id direction))))))
      ;; _NET_REQUEST_FRAME_EXTENTS
      ((= type xcb:Atom:_NET_REQUEST_FRAME_EXTENTS)
       (let ((buffer (exwm--id->buffer id))
@@ -645,7 +673,7 @@
                             ;; xcb:Atom:_NET_WM_STATE_SHADED
                             ;; xcb:Atom:_NET_WM_STATE_SKIP_TASKBAR
                             ;; xcb:Atom:_NET_WM_STATE_SKIP_PAGER
-                            ;; xcb:Atom:_NET_WM_STATE_HIDDEN
+                            xcb:Atom:_NET_WM_STATE_HIDDEN
                             xcb:Atom:_NET_WM_STATE_FULLSCREEN
                             ;; xcb:Atom:_NET_WM_STATE_ABOVE
                             ;; xcb:Atom:_NET_WM_STATE_BELOW
@@ -702,10 +730,11 @@
                        :visual 0
                        :value-mask xcb:CW:OverrideRedirect
                        :override-redirect 1))
-    ;; Set _NET_WM_NAME
+    ;; Set _NET_WM_NAME.  Must be set to the name of the window manager, as
+    ;; required by wm-spec.
     (xcb:+request exwm--connection
         (make-instance 'xcb:ewmh:set-_NET_WM_NAME
-                       :window new-id :data "EXWM: exwm--guide-window"))
+                       :window new-id :data "EXWM"))
     (dolist (i (list exwm--root new-id))
       ;; Set _NET_SUPPORTING_WM_CHECK
       (xcb:+request exwm--connection
